@@ -6,15 +6,13 @@ use Carp;
 use POSIX qw(strftime);
 use File::BaseDir qw/
 	xdg_config_home xdg_config_files
-	xdg_data_home xdg_data_dirs xdg_data_files
-	xdg_cache_home /;
+	xdg_data_home xdg_data_dirs xdg_data_files /;
 use File::MimeInfo::Magic;
 use Gtk2;
 use Gtk2::Gdk::Keysyms;
 use Gtk2::SimpleList;
 use Zim;
 use Zim::Events;
-use Zim::History;
 use Zim::GUI::Component;
 use Zim::GUI::PathBar;
 use Zim::GUI::PageView;
@@ -24,7 +22,7 @@ my $has_mimeapplications = $@ ? 0 : 1;
 warn "WARNING: Could not use 'File::MimeInfo::Applications', disabling application bindings\n"
 	unless $has_mimeapplications;
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 our @ISA = qw/Zim::Events Zim::GUI::Component/;
 
 =head1 NAME
@@ -82,11 +80,9 @@ browser
 file_browser
 email_client
 date_format	%A %d/%m/%Y
-hist_max	20
 undo_max	50
 follow_new_link	0
 backsp_unindent	1
-use_xdg_cache	0
 show_spell	1
 spell_language
 cal_namespace	:Date:
@@ -318,7 +314,11 @@ sub init {
 	
 	# load repository
 	my $repository = $self->{repository}
-		|| Zim->new(dir => $$self{dir}, type => $$self{type});
+		|| Zim->new(
+			dir      => $$self{dir},
+			type     => $$self{type},
+		);
+	$self->{history} = $repository->init_history();
 	my $dir = $repository->{dir}; # FIXME what if there is no dir
 	$self->{name} ||= $repository->config->{name};
 	unless ($self->{name}) { # needed for trayicon tooltip
@@ -493,29 +493,9 @@ sub gui_init {
 
 	$vbox->set_focus_chain($l_vbox, $page_view->widget);
 		# get pathbar and toolbar out of the loop
-
-	# FIXME shouldn't the repository manage this directory ?
-	if ($self->{settings}{use_xdg_cache}) {
-		my $root = File::Spec->rel2abs($self->{repository}{dir});
-		my ($vol, $dirs) = File::Spec->splitpath($root, 'NO_FILE');
-		my @dirs = File::Spec->splitdir($dirs);
-		$self->{hist_file} ||= File::Spec->catfile( 
-			xdg_cache_home, 'zim', join ':', $vol, @dirs );
-	}
-	else {
-		my $root = File::Spec->rel2abs($self->{repository}{dir});
-		$self->{hist_file} ||= File::Spec->catfile($root, '.zim.history.cache'); 
-	}
 	
-	eval {
-		$self->{hist_file} = undef
-			unless -w $self->{hist_file}
-			and    -w Zim::File->dir($self->{hist_file}) ;
-		my $history = Zim::History->new(
-			$self->{hist_file}, $page_view, $self->{settings}{hist_max} );
-		$self->{objects}{History} = $history;
-	};
-	$self->exit_error($@) if $@;
+	my $hist = $self->{history};
+	$hist->set_GUI($page_view) if $hist;
 
 	## Some wiring
 	my $accels = $ui->get_accel_group;
@@ -539,16 +519,15 @@ sub gui_init {
 	$accels->connect( Gtk2::Accelerator->parse('F5'), ['visible'],
 		sub { $self->Reload } );
 
-	
 	$self->signal_connect('page_loaded', sub {
 		my $self = shift;
-		my $state = $self->History->get_state;
+		my $state = $hist->get_state;
 		$self->actions_set_sensitive(
 			GoBack     => ($$state{back} ? 1 : 0),
 			GoForward  => ($$state{forw} ? 1 : 0),
 			GoParent   => ($self->{page}->namespace =~ /[^:]/),
 		)
-	} );
+	} ) if $hist;
 	$self->signal_connect('page_loaded', sub {
 		my $self = shift;
 		$self->actions_set_sensitive(
@@ -617,9 +596,10 @@ sub gui_show {
 #	$window->move($x,$y) if defined($x) and defined ($y);
 	
 	## Find a page to load
+	my $hist = $self->{history};
 	unless ($self->{page}) {
-		my $page = $self->{objects}{History}->get_current
-			|| $self->{home};
+		my $page = $hist ? $hist->get_current : undef;
+		$page ||= $self->{home};
 		$page = ref($page)
 			? $page
 			: $self->{repository}->resolve_page($page);
@@ -753,13 +733,7 @@ sub on_delete_event {
 	$self->save_page() || return 1;
 	$self->save_config;
 
-	unless ($self->{settings}{read_only}) {
-		my ($vol, $dirs, undef) =
-			File::Spec->splitpath($self->{hist_file});
-		my $dir = File::Spec->catpath($vol, $dirs);
-		$self->{objects}{History}->write if -d $dir;
-			# Only safe history if directory exists ..
-	}
+	$self->{history}->write if $self->{history};
 
 	return 0;
 }
@@ -1143,7 +1117,8 @@ sub _load_page { # load _without_ saving first
 	# if we did get a record it is from GoBack() or GoForw()
 	# and the history is updated already
 	unless ($rec) { # get history record
-		$rec = $self->{objects}{History}->set_current($name);
+		$rec = $self->{history}->set_current($name)
+			if $self->{history};
 	}
 	$self->{page} = $page;
 
@@ -1157,14 +1132,17 @@ sub _load_page { # load _without_ saving first
 	# Update various objects
 	$self->{objects}{PageView}->set_state($rec->{state})
 		if $rec and $rec->{state};
-	$self->{objects}{History}->delete_recent($prevpage->name)
-		unless !defined $prevpage or $prevpage->exists;
+	$self->{history}->delete_recent($prevpage->name)
+		if   $self->{history}
+		and (defined $prevpage and ! $prevpage->exists);
 	$self->signal_emit('page_loaded', $name);
 	
 	$self->{window}->set_title("$name - Zim");
 	$self->update_status();
 	my $links = scalar $page->list_backlinks;
 	$self->{statusl}->set_text_with_mnemonic("  $links _Back links");
+
+	$self->{history}->write if $self->{history};
 }
 
 =item C<load_date(DAY, MONTH, YEAR)>
@@ -1251,7 +1229,6 @@ sub Save {
 	my $self = shift;
 	$self->save_page('FORCE');
 	$self->save_config;
-	$self->History->write;
 }
 
 =item C<save_page(FORCE)>
@@ -1456,6 +1433,7 @@ sub RenamePage {
 	}
 	
 	warn "# moved $from => $to\n";
+	$self->{history}->delete_recent("$from") if $self->{history};
 	$self->signal_emit('page_renamed', $from => $to);
 	$self->load_page($to) if $move_current;
 	
@@ -1495,7 +1473,7 @@ sub DeletePage {
 	return $self->error_dialog("Could not delete $name\n\n$@") if $@;
 
 	# TODO trigger this with signals
-	$self->{objects}{History}->delete_recent($name);
+	$self->{history}->delete_recent($name) if $self->{history};
 	$self->signal_emit('page_deleted', $name);
 
 	$self->load_page($page) if $delete_current;
@@ -1760,14 +1738,16 @@ Go forward one or more steps in the history stack.
 sub GoBack {
 	my $self = shift;
 	my $i = shift || 1;
-	my $rec = $self->{objects}{History}->back($i) || return;
+	return unless $self->{history};
+	my $rec = $self->{history}->back($i) || return;
 	$self->load_page($rec);
 }
 
 sub GoForward {
 	my $self = shift;
 	my $i = shift || 1;
-	my $rec = $self->{objects}{History}->forw($i) || return;
+	return unless $self->{history};
+	my $rec = $self->{history}->forw($i) || return;
 	$self->load_page($rec);
 }
 
@@ -1791,7 +1771,8 @@ sub GoParent {
 
 sub GoChild {
 	my $self = shift;
-	my $namespace = $self->History->get_namespace;
+	return unless $self->{history};
+	my $namespace = $self->{history}->get_namespace;
 	my $name = $self->{page}->name;
 	return unless $namespace =~ /^(:*\Q$name\E:+[^:]+)/;
 	$self->load_page($1);
@@ -1875,8 +1856,34 @@ This dialog tells you about the version of zim you are using.
 
 =cut
 
+
+=begin comment
+
+sub About {
+	# For Gtk > 2.6.0
+	my $self = shift;
+	my %info = (
+		name => 'Zim',
+		version => $Zim::VERSION,
+		copyright => $Zim::COPYRIGHT,
+		license => $Zim::LONG_VERSION,
+		authors => $Zim::AUTHORS,
+		logo => Gtk2::Gdk::Pixbuf->new_from_file($Zim::ICON),
+		# logo-icon-name => 'zim',
+		comments => 'A desktop wiki',
+		website => $Zim::WEBSITE,
+	);
+	my $hook = sub {$self->link_clicked($_[1])};
+	Gtk2::AboutDialog->set_url_hook($hook);
+	Gtk2::AboutDialog->set_email_hook($hook);
+	Gtk2->show_about_dialog($self->{window}, %info);
+}
+
+=end comment
+
+=cut
+
 sub About {
-	# TODO use Gtk2::AboutDialog for gtk > 2.6
 	my $self = shift;
 	
 	my $dialog = Gtk2::Dialog->new(
