@@ -4,7 +4,7 @@ use strict;
 use Gtk2;
 use Zim::GUI::Component;
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 our @ISA = qw/Zim::GUI::Component/;
 
 our $icon_class;
@@ -146,9 +146,6 @@ sub popup_menu {
 
 	my $menu = Gtk2::Menu->new();
 
-	my $hist = $self->{app}->History;
-	my ($idx, @recent) = $hist->get_recent;
-
 	for (qw/Home Today/) {
 		my $item = Gtk2::ImageMenuItem->new_with_label($_);
 		$item->set_image(
@@ -172,6 +169,8 @@ sub popup_menu {
 	$separator = Gtk2::SeparatorMenuItem->new();
 	$menu->add($separator);
 	
+	my $hist = $self->{app}{history};
+	my ($idx, @recent) = $hist ? $hist->get_recent : ();
 	for (reverse 0 .. $#recent) {
 		my $rec = $recent[$_];
 		my $item = Gtk2::ImageMenuItem->new_with_label($rec->{basename});
